@@ -20,7 +20,7 @@
 use adw::subclass::prelude::*;
 use anyhow::Result;
 use async_channel::{Receiver, Sender};
-use gettextrs::{bindtextdomain, setlocale, textdomain, LocaleCategory};
+use gettextrs::{LocaleCategory, bindtextdomain, setlocale, textdomain};
 use glib::clone;
 use gtk::prelude::*;
 use gtk::{gio, glib};
@@ -35,6 +35,7 @@ use crate::feed_manager::FeedManager;
 use crate::i18n::i18n;
 use crate::settings;
 use crate::utils;
+use crate::widgets::player::StreamMode;
 use crate::widgets::show_menu::{mark_all_notif, remove_show_notif};
 use crate::widgets::{EpisodeDescription, SearchResults, ShowWidget};
 use crate::window::MainWindow;
@@ -333,6 +334,7 @@ impl PdApplication {
             }
             Action::GoToEpisodeDescription(show, ep) => {
                 let description_widget = EpisodeDescription::new(ep, show, window.sender().clone());
+                window.pop_episode_description();
                 window.push_page(&description_widget);
             }
             Action::GoToShow(pd) => {
@@ -423,15 +425,15 @@ impl PdApplication {
                 FeedManager::refresh_done(data.sender.clone(), id);
             }
             Action::InitEpisode(id) => {
-                let res = window.init_episode(id, None, false);
+                let res = window.init_episode(id, None, StreamMode::LocalOnly);
                 debug_assert!(res.is_ok());
             }
             Action::InitEpisodeAt(id, second) => {
-                let res = window.init_episode(id, Some(second), false);
+                let res = window.init_episode(id, Some(second), StreamMode::StreamFallback);
                 debug_assert!(res.is_ok());
             }
             Action::StreamEpisode(id) => {
-                let res = window.init_episode(id, None, true);
+                let res = window.init_episode(id, None, StreamMode::StreamOnly);
                 debug_assert!(res.is_ok());
             }
             Action::UpdateMprisCover(id, dl_success) => {
@@ -493,7 +495,9 @@ impl PdApplication {
         textdomain("gnome-podcasts").expect("Unable to switch to the text domain");
 
         // Make sure the app icon shows up in PulseAudio settings
-        env::set_var("PULSE_PROP_application.icon_name", APP_ID);
+        unsafe {
+            env::set_var("PULSE_PROP_application.icon_name", APP_ID);
+        }
 
         let application = Self::new();
 
