@@ -17,6 +17,7 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+use adw::prelude::*;
 use anyhow::Result;
 use async_channel::Sender;
 use chrono::prelude::*;
@@ -25,7 +26,6 @@ use gio::File;
 use glib::clone;
 use glib::{SignalHandlerId, WeakRef};
 use gst::ClockTime;
-use gtk::prelude::*;
 use gtk::{gio, glib};
 use mpris_server::{Metadata, PlaybackStatus, Player};
 use once_cell::sync::Lazy;
@@ -41,7 +41,7 @@ use crate::config::APP_ID;
 use crate::download_covers::load_widget_texture;
 use crate::i18n::i18n;
 use podcasts_data::{
-    dbqueries, EpisodeId, EpisodeModel, EpisodeWidgetModel, ShowCoverModel, ShowId, USER_AGENT,
+    EpisodeId, EpisodeModel, EpisodeWidgetModel, ShowCoverModel, ShowId, USER_AGENT, dbqueries,
 };
 
 #[derive(Debug, Clone, Copy)]
@@ -70,7 +70,7 @@ struct PlayerInfo {
     episode_small: gtk::Label,
     cover_small: gtk::Image,
     mpris: Option<Rc<Player>>,
-    restore_position: i32,
+    restore_position: Option<i32>,
     finished_restore: bool,
     ep: Option<EpisodeWidgetModel>,
     episode_id: RefCell<Option<EpisodeId>>,
@@ -104,6 +104,11 @@ impl PlayerInfo {
         let mut metadata = Metadata::new();
         metadata.set_artist(Some(vec![podcast.title().to_string()]));
         metadata.set_title(Some(episode.title().to_string()));
+        metadata.set_length(
+            episode
+                .duration()
+                .map(|s| mpris_server::Time::from_secs(s as i64)),
+        );
 
         // Set the cover if it is already cached.
         let art_path = crate::download_covers::determin_cover_path(podcast, None);
@@ -218,6 +223,12 @@ impl PlayerInfo {
         load_widget_texture(&self.cover, show.id(), crate::Thumb64);
         load_widget_texture(&self.cover_small, show.id(), crate::Thumb64);
     }
+
+    fn update_mpris_position(&self, position: Position) -> Option<()> {
+        let time = mpris_server::Time::from_secs(position.seconds() as i64);
+        self.mpris.as_ref()?.set_position(time);
+        Some(())
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -372,6 +383,7 @@ pub(crate) struct PlayerSheet {
     rate: PlayerRate,
     show: gtk::Label,
     episode: gtk::Label,
+    go_to_episode: gtk::Button,
 }
 
 impl PlayerSheet {
@@ -388,11 +400,25 @@ impl PlayerSheet {
         let slider = builder.object("slider").unwrap();
         let rewind = builder.object("rewind").unwrap();
         let forward = builder.object("forward").unwrap();
-        let bottom: gtk::Box = builder.object("bottom").unwrap();
+        let rate_container: adw::Bin = builder.object("rate_container").unwrap();
         let show = builder.object("show_label").unwrap();
         let episode = builder.object("episode_label").unwrap();
+        let go_to_episode: gtk::Button = builder.object("go_to_episode").unwrap();
 
-        bottom.prepend(&rate.btn);
+        rate_container.set_child(Some(&rate.btn));
+
+        go_to_episode.connect_clicked(|button| {
+            let app = gio::Application::default()
+                .expect("Could not get default application")
+                .downcast::<gtk::Application>()
+                .unwrap();
+
+            app.activate_action("go-to-episode", button.action_target_value().as_ref());
+            let window = app.active_window().expect("No active window");
+            if let Err(e) = window.activate_action("win.close-bottom-sheet", None) {
+                warn!("Failed to win.close-bottom-sheet {e}");
+            }
+        });
 
         PlayerSheet {
             sheet,
@@ -408,6 +434,7 @@ impl PlayerSheet {
             rate,
             show,
             episode,
+            go_to_episode,
         }
     }
 
@@ -415,6 +442,8 @@ impl PlayerSheet {
         self.episode.set_text(episode.title());
         self.show.set_text(show.title());
         load_widget_texture(&self.cover, show.id(), crate::Thumb256);
+        self.go_to_episode
+            .set_action_target(Some::<gtk::glib::Variant>(episode.id().0.into()));
     }
 }
 
@@ -537,7 +566,7 @@ impl Default for PlayerWidget {
             show_small,
             episode_small,
             cover_small,
-            restore_position: 0,
+            restore_position: None,
             finished_restore: false,
             episode_id: RefCell::new(None),
         };
@@ -571,6 +600,13 @@ impl Default for PlayerWidget {
     }
 }
 
+#[derive(PartialEq)]
+pub enum StreamMode {
+    LocalOnly,
+    StreamOnly,
+    StreamFallback,
+}
+
 impl PlayerWidget {
     fn on_rate_changed(&self, rate: f64) {
         self.set_playback_rate(rate);
@@ -586,7 +622,7 @@ impl PlayerWidget {
         &mut self,
         sender: &Sender<Action>,
         id: EpisodeId,
-        stream: bool,
+        stream: StreamMode,
         second: Option<i32>,
     ) -> Result<()> {
         let ep = dbqueries::get_episode_widget_from_id(id)?;
@@ -594,11 +630,18 @@ impl PlayerWidget {
 
         self.sheet.initialize_episode(&ep, &pd);
 
-        self.info.restore_position = second.unwrap_or(ep.play_position());
+        self.info.restore_position = second.or_else(|| {
+            let episode_position = ep.play_position();
+            if episode_position == 0 {
+                None
+            } else {
+                Some(episode_position)
+            }
+        });
         self.info.finished_restore = false;
         self.info.init(sender, &ep, &pd);
 
-        if stream {
+        if stream == StreamMode::StreamOnly {
             if let Some(uri) = ep.uri() {
                 self.init_uri(uri, second);
                 return Ok(());
@@ -618,18 +661,23 @@ impl PlayerWidget {
             } else {
                 error!("failed to create path for episode {:#?}", ep);
             }
+        } else if stream == StreamMode::StreamFallback {
+            if let Some(uri) = ep.uri() {
+                self.init_uri(uri, second);
+                return Ok(());
+            } else {
+                error!("No uri for episode");
+            }
         } else {
             error!("Episode not downloaded yet.");
         }
 
-        // FIXME: Stream stuff
-        // unimplemented!()
         Ok(())
     }
 
     fn init_uri(&mut self, uri: &str, second: Option<i32>) {
         // If it's not the same file load the uri, otherwise just unpause
-        if self.player.uri().map_or(true, |s| s != uri) {
+        if self.player.uri().is_none_or(|s| s != uri) {
             self.player.set_uri(Some(uri));
         } else if second.is_some() {
             // force a jump now if already playing and a jump is given
@@ -692,8 +740,8 @@ impl PlayerWidget {
     /// Returns Some(()) if the restore was successful and None otherwise.
     fn restore_play_position(&self) -> Option<()> {
         let pos = self.info.restore_position;
-        let s: u64 = pos.try_into().ok()?;
-        if pos != 0 {
+        if let Some(pos) = pos {
+            let s: u64 = pos.try_into().ok()?;
             self.player.seek(ClockTime::from_seconds(s));
             Some(())
         } else {
@@ -805,7 +853,7 @@ impl PlayerExt for PlayerWidget {
         }
 
         self.info.ep = None;
-        self.info.restore_position = 0;
+        self.info.restore_position = None;
         self.player.stop();
         if let Some(mpris) = self.info.mpris.as_ref() {
             crate::MAINCONTEXT.spawn_local_with_priority(
@@ -1121,7 +1169,8 @@ impl PlayerWrapper {
                                 Ok(())
                             }
                         });
-                        player_widget.borrow().timer.on_position_updated(pos)
+                        player_widget.borrow().timer.on_position_updated(pos);
+                        player_widget.borrow().info.update_mpris_position(pos);
                     }
                 }
             }
